@@ -1,8 +1,8 @@
 #include <stdlib.h>
 #include <libebook/e-book.h>
 
-int
-main (int argc, char **argv)
+gint
+main (gint argc, gchar **argv)
 {
 	EContact *contact;
 	EContactDate date, *dp;
@@ -27,6 +27,6 @@ main (int argc, char **argv)
 	  printf ("failed\n");
 	else
 	  printf ("passed\n");
-	
+
 	return 0;
 }
