@@ -37,20 +37,20 @@
 
 #define d(x)
 
-static gboolean construct (CamelService *service, CamelSession *session, CamelProvider *provider, CamelURL *url, CamelException *ex);
-static CamelFolder *get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, CamelException * ex);
-static gchar *get_name(CamelService *service, gboolean brief);
-static CamelFolder *local_get_inbox (CamelStore *store, CamelException *ex);
-static CamelFolder *local_get_junk(CamelStore *store, CamelException *ex);
-static CamelFolder *local_get_trash(CamelStore *store, CamelException *ex);
-static CamelFolderInfo *get_folder_info (CamelStore *store, const gchar *top, guint32 flags, CamelException *ex);
-static gboolean delete_folder(CamelStore *store, const gchar *folder_name, CamelException *ex);
-static gboolean rename_folder(CamelStore *store, const gchar *old, const gchar *new, CamelException *ex);
-static CamelFolderInfo *create_folder(CamelStore *store, const gchar *parent_name, const gchar *folder_name, CamelException *ex);
-static gboolean local_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelException *ex);
+static gboolean construct (CamelService *service, CamelSession *session, CamelProvider *provider, CamelURL *url, GError **error);
+static CamelFolder *local_store_get_folder_sync (CamelStore *store, const gchar *folder_name, CamelStoreGetFolderFlags flags, GCancellable *cancellable, GError **error);
+static gchar *get_name (CamelService *service, gboolean brief);
+static CamelFolder *local_store_get_inbox_folder_sync (CamelStore *store, GCancellable *cancellable, GError **error);
+static CamelFolder *local_store_get_junk_folder_sync (CamelStore *store, GCancellable *cancellable, GError **error);
+static CamelFolder *local_store_get_trash_folder_sync (CamelStore *store, GCancellable *cancellable, GError **error);
+static CamelFolderInfo *local_store_get_folder_info_sync (CamelStore *store, const gchar *top, CamelStoreGetFolderInfoFlags flags, GCancellable *cancellable, GError **error);
+static gboolean local_store_delete_folder_sync (CamelStore *store, const gchar *folder_name, GCancellable *cancellable, GError **error);
+static gboolean local_store_rename_folder_sync (CamelStore *store, const gchar *old, const gchar *new, GCancellable *cancellable, GError **error);
+static CamelFolderInfo *local_store_create_folder_sync (CamelStore *store, const gchar *parent_name, const gchar *folder_name, GCancellable *cancellable, GError **error);
+static gboolean local_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, GError **error);
 
-static gchar *local_get_full_path(CamelLocalStore *lf, const gchar *full_name);
-static gchar *local_get_meta_path(CamelLocalStore *lf, const gchar *full_name, const gchar *ext);
+static gchar *local_get_full_path (CamelLocalStore *lf, const gchar *full_name);
+static gchar *local_get_meta_path (CamelLocalStore *lf, const gchar *full_name, const gchar *ext);
 
 G_DEFINE_TYPE (CamelLocalStore, camel_local_store, CAMEL_TYPE_STORE)
 
@@ -80,16 +80,16 @@ camel_local_store_class_init (CamelLocalStoreClass *class)
 	service_class->get_name = get_name;
 
 	store_class = CAMEL_STORE_CLASS (class);
-	store_class->get_folder = get_folder;
-	store_class->get_inbox = local_get_inbox;
-	store_class->get_trash = local_get_trash;
-	store_class->get_junk = local_get_junk;
-	store_class->get_folder_info = get_folder_info;
-	store_class->free_folder_info = camel_store_free_folder_info_full;
-	store_class->create_folder = create_folder;
-	store_class->delete_folder = delete_folder;
-	store_class->rename_folder = rename_folder;
 	store_class->can_refresh_folder = local_can_refresh_folder;
+	store_class->free_folder_info = camel_store_free_folder_info_full;
+	store_class->get_folder_sync = local_store_get_folder_sync;
+	store_class->get_folder_info_sync = local_store_get_folder_info_sync;
+	store_class->get_inbox_folder_sync = local_store_get_inbox_folder_sync;
+	store_class->get_junk_folder_sync = local_store_get_junk_folder_sync;
+	store_class->get_trash_folder_sync = local_store_get_trash_folder_sync;
+	store_class->create_folder_sync = local_store_create_folder_sync;
+	store_class->delete_folder_sync = local_store_delete_folder_sync;
+	store_class->rename_folder_sync = local_store_rename_folder_sync;
 
 	class->get_full_path = local_get_full_path;
 	class->get_meta_path = local_get_meta_path;
@@ -105,7 +105,7 @@ construct (CamelService *service,
            CamelSession *session,
            CamelProvider *provider,
            CamelURL *url,
-           CamelException *ex)
+           GError **error)
 {
 	CamelLocalStore *local_store = CAMEL_LOCAL_STORE (service);
 	CamelServiceClass *service_class;
@@ -113,7 +113,7 @@ construct (CamelService *service,
 
 	/* Chain up to parent's construct() method. */
 	service_class = CAMEL_SERVICE_CLASS (camel_local_store_parent_class);
-	if (!service_class->construct (service, session, provider, url, ex))
+	if (!service_class->construct (service, session, provider, url, error))
 		return FALSE;
 
 	len = strlen (service->url->path);
@@ -132,27 +132,33 @@ camel_local_store_get_toplevel_dir (CamelLocalStore *store)
 }
 
 static CamelFolder *
-get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, CamelException * ex)
+local_store_get_folder_sync (CamelStore *store,
+                             const gchar *folder_name,
+                             CamelStoreGetFolderFlags flags,
+                             GCancellable *cancellable,
+                             GError **error)
 {
-	gint len = strlen(((CamelLocalStore *)store)->toplevel_dir);
-	gchar *path = g_alloca(len + 1);
+	gint len = strlen (((CamelLocalStore *)store)->toplevel_dir);
+	gchar *path = g_alloca (len + 1);
 	struct stat st;
 
-	strcpy(path, ((CamelLocalStore *)store)->toplevel_dir);
-	if (G_IS_DIR_SEPARATOR(path[len-1]))
+	strcpy (path, ((CamelLocalStore *)store)->toplevel_dir);
+	if (G_IS_DIR_SEPARATOR (path[len-1]))
 		path[len-1] = '\0';
 
-	if (!g_path_is_absolute(path)) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+	if (!g_path_is_absolute (path)) {
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_STORE_ERROR_NO_FOLDER,
 			_("Store root %s is not an absolute path"), path);
 		return NULL;
 	}
 
-	if (g_stat(path, &st) == 0) {
-		if (!S_ISDIR(st.st_mode)) {
-			camel_exception_setv (
-				ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+	if (g_stat (path, &st) == 0) {
+		if (!S_ISDIR (st.st_mode)) {
+			g_set_error (
+				error, CAMEL_STORE_ERROR,
+				CAMEL_STORE_ERROR_NO_FOLDER,
 				_("Store root %s is not a regular directory"), path);
 			return NULL;
 		}
@@ -161,8 +167,9 @@ get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, CamelExc
 
 	if (errno != ENOENT
 	    || (flags & CAMEL_STORE_FOLDER_CREATE) == 0) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
 			_("Cannot get folder: %s: %s"),
 			path, g_strerror (errno));
 		return NULL;
@@ -170,8 +177,9 @@ get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, CamelExc
 
 	/* need to create the dir heirarchy */
 	if (g_mkdir_with_parents (path, 0700) == -1 && errno != EEXIST) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
 			_("Cannot get folder: %s: %s"),
 			path, g_strerror (errno));
 		return NULL;
@@ -181,30 +189,35 @@ get_folder(CamelStore * store, const gchar *folder_name, guint32 flags, CamelExc
 }
 
 static CamelFolder *
-local_get_inbox(CamelStore *store, CamelException *ex)
+local_store_get_inbox_folder_sync (CamelStore *store,
+                                   GCancellable *cancellable,
+                                   GError **error)
 {
-	camel_exception_set (
-		ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+	g_set_error (
+		error, CAMEL_STORE_ERROR,
+		CAMEL_STORE_ERROR_NO_FOLDER,
 		_("Local stores do not have an inbox"));
 
 	return NULL;
 }
 
 static CamelFolder *
-local_get_trash (CamelStore *store,
-                 CamelException *ex)
+local_store_get_trash_folder_sync (CamelStore *store,
+                                   GCancellable *cancellable,
+                                   GError **error)
 {
 	CamelFolder *folder;
 
-	/* Chain up to parent's get_trash() method. */
-	folder = CAMEL_STORE_CLASS (camel_local_store_parent_class)->get_trash (store, ex);
+	/* Chain up to parent's get_trash_folder_sync() method. */
+	folder = CAMEL_STORE_CLASS (camel_local_store_parent_class)->
+		get_trash_folder_sync (store, cancellable, error);
 
 	if (folder) {
 		CamelObject *object = CAMEL_OBJECT (folder);
 		gchar *state = camel_local_store_get_meta_path(store, CAMEL_VTRASH_NAME, ".cmeta");
 
 		camel_object_set_state_filename (object, state);
-		g_free(state);
+		g_free (state);
 		/* no defaults? */
 		camel_object_state_read (object);
 	}
@@ -213,20 +226,22 @@ local_get_trash (CamelStore *store,
 }
 
 static CamelFolder *
-local_get_junk (CamelStore *store,
-                CamelException *ex)
+local_store_get_junk_folder_sync (CamelStore *store,
+                                  GCancellable *cancellable,
+                                  GError **error)
 {
 	CamelFolder *folder;
 
-	/* Chain up to parent's get_junk() method. */
-	folder = CAMEL_STORE_CLASS (camel_local_store_parent_class)->get_junk (store, ex);
+	/* Chain up to parent's get_junk_folder_sync() method. */
+	folder = CAMEL_STORE_CLASS (camel_local_store_parent_class)->
+		get_junk_folder_sync (store, cancellable, error);
 
 	if (folder) {
 		CamelObject *object = CAMEL_OBJECT (folder);
 		gchar *state = camel_local_store_get_meta_path(store, CAMEL_VJUNK_NAME, ".cmeta");
 
 		camel_object_set_state_filename (object, state);
-		g_free(state);
+		g_free (state);
 		/* no defaults? */
 		camel_object_state_read (object);
 	}
@@ -246,8 +261,11 @@ get_name (CamelService *service, gboolean brief)
 }
 
 static CamelFolderInfo *
-get_folder_info (CamelStore *store, const gchar *top,
-		 guint32 flags, CamelException *ex)
+local_store_get_folder_info_sync (CamelStore *store,
+                                  const gchar *top,
+                                  CamelStoreGetFolderInfoFlags flags,
+                                  GCancellable *cancellable,
+                                  GError **error)
 {
 	/* FIXME: This is broken, but it corresponds to what was
 	 * there before.
@@ -259,10 +277,11 @@ get_folder_info (CamelStore *store, const gchar *top,
 }
 
 static CamelFolderInfo *
-create_folder (CamelStore *store,
-               const gchar *parent_name,
-               const gchar *folder_name,
-               CamelException *ex)
+local_store_create_folder_sync (CamelStore *store,
+                                const gchar *parent_name,
+                                const gchar *folder_name,
+                                GCancellable *cancellable,
+                                GError **error)
 {
 	gchar *path = ((CamelLocalStore *)store)->toplevel_dir;
 	gchar *name;
@@ -272,9 +291,10 @@ create_folder (CamelStore *store,
 
 	/* This is a pretty hacky version of create folder, but should basically work */
 
-	if (!g_path_is_absolute(path)) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+	if (!g_path_is_absolute (path)) {
+		g_set_error (
+			error, CAMEL_STORE_ERROR,
+			CAMEL_STORE_ERROR_NO_FOLDER,
 			_("Store root %s is not an absolute path"), path);
 		return NULL;
 	}
@@ -284,31 +304,32 @@ create_folder (CamelStore *store,
 	else
 		name = g_strdup_printf("%s/%s", path, folder_name);
 
-	if (g_stat(name, &st) == 0 || errno != ENOENT) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+	if (g_stat (name, &st) == 0 || errno != ENOENT) {
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
 			_("Cannot get folder: %s: %s"),
 			name, g_strerror (errno));
-		g_free(name);
+		g_free (name);
 		return NULL;
 	}
 
-	g_free(name);
+	g_free (name);
 
 	if (parent_name)
 		name = g_strdup_printf("%s/%s", parent_name, folder_name);
 	else
 		name = g_strdup_printf("%s", folder_name);
 
-	folder = CAMEL_STORE_GET_CLASS (store)->get_folder (
-		store, name, CAMEL_STORE_FOLDER_CREATE, ex);
+	folder = CAMEL_STORE_GET_CLASS (store)->get_folder_sync (
+		store, name, CAMEL_STORE_FOLDER_CREATE, cancellable, error);
 	if (folder) {
 		g_object_unref (folder);
-		info = CAMEL_STORE_GET_CLASS (store)->get_folder_info (
-			store, name, 0, ex);
+		info = CAMEL_STORE_GET_CLASS (store)->get_folder_info_sync (
+			store, name, 0, cancellable, error);
 	}
 
-	g_free(name);
+	g_free (name);
 
 	return info;
 }
@@ -319,17 +340,17 @@ xrename (const gchar *oldp,
          const gchar *prefix,
          const gchar *suffix,
          gint missingok,
-         CamelException *ex)
+         GError **error)
 {
 	struct stat st;
-	gchar *old = g_strconcat(prefix, oldp, suffix, NULL);
-	gchar *new = g_strconcat(prefix, newp, suffix, NULL);
+	gchar *old = g_strconcat (prefix, oldp, suffix, NULL);
+	gchar *new = g_strconcat (prefix, newp, suffix, NULL);
 	gint ret = -1;
 	gint err = 0;
 
 	d(printf("renaming %s%s to %s%s\n", oldp, suffix, newp, suffix));
 
-	if (g_stat(old, &st) == -1) {
+	if (g_stat (old, &st) == -1) {
 		if (missingok && errno == ENOENT) {
 			ret = 0;
 		} else {
@@ -337,7 +358,7 @@ xrename (const gchar *oldp,
 			ret = -1;
 		}
 	} else if ((!g_file_test (new, G_FILE_TEST_EXISTS) || g_remove (new) == 0) &&
-		   g_rename(old, new) == 0) {
+		   g_rename (old, new) == 0) {
 		ret = 0;
 	} else {
 		err = errno;
@@ -345,23 +366,25 @@ xrename (const gchar *oldp,
 	}
 
 	if (ret == -1) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (err),
 			_("Could not rename folder %s to %s: %s"),
 			old, new, g_strerror (err));
 	}
 
-	g_free(old);
-	g_free(new);
+	g_free (old);
+	g_free (new);
 	return ret;
 }
 
 /* default implementation, rename all */
 static gboolean
-rename_folder(CamelStore *store,
-              const gchar *old,
-              const gchar *new,
-              CamelException *ex)
+local_store_rename_folder_sync (CamelStore *store,
+                                const gchar *old,
+                                const gchar *new,
+                                GCancellable *cancellable,
+                                GError **error)
 {
 	gchar *path = CAMEL_LOCAL_STORE (store)->toplevel_dir;
 	CamelLocalFolder *folder = NULL;
@@ -372,30 +395,30 @@ rename_folder(CamelStore *store,
 
 	d(printf("local rename folder '%s' '%s'\n", old, new));
 
-	folder = camel_object_bag_get(store->folders, old);
+	folder = camel_object_bag_get (store->folders, old);
 	if (folder && folder->index) {
-		if (camel_index_rename(folder->index, newibex) == -1)
+		if (camel_index_rename (folder->index, newibex) == -1)
 			goto ibex_failed;
 	} else {
 		/* TODO: camel_text_index_rename should find out if we have an active index itself? */
-		if (camel_text_index_rename(oldibex, newibex) == -1)
+		if (camel_text_index_rename (oldibex, newibex) == -1)
 			goto ibex_failed;
 	}
 
-	if (xrename(old, new, path, ".ev-summary", TRUE, ex))
+	if (xrename(old, new, path, ".ev-summary", TRUE, error))
 		goto summary_failed;
 
-	if (xrename(old, new, path, ".ev-summary-meta", TRUE, ex))
+	if (xrename(old, new, path, ".ev-summary-meta", TRUE, error))
 		goto summary_failed;
 
-	if (xrename(old, new, path, ".cmeta", TRUE, ex))
+	if (xrename(old, new, path, ".cmeta", TRUE, error))
 		goto cmeta_failed;
 
-	if (xrename(old, new, path, "", FALSE, ex))
+	if (xrename(old, new, path, "", FALSE, error))
 		goto base_failed;
 
-	g_free(newibex);
-	g_free(oldibex);
+	g_free (newibex);
+	g_free (oldibex);
 
 	if (folder)
 		g_object_unref (folder);
@@ -405,25 +428,26 @@ rename_folder(CamelStore *store,
 	/* The (f)utility of this recovery effort is quesitonable */
 
 base_failed:
-	xrename(new, old, path, ".cmeta", TRUE, ex);
+	xrename(new, old, path, ".cmeta", TRUE, NULL);
 
 cmeta_failed:
-	xrename(new, old, path, ".ev-summary", TRUE, ex);
-	xrename(new, old, path, ".ev-summary-meta", TRUE, ex);
+	xrename(new, old, path, ".ev-summary", TRUE, NULL);
+	xrename(new, old, path, ".ev-summary-meta", TRUE, NULL);
 summary_failed:
 	if (folder) {
 		if (folder->index)
-			camel_index_rename(folder->index, oldibex);
+			camel_index_rename (folder->index, oldibex);
 	} else
-		camel_text_index_rename(newibex, oldibex);
+		camel_text_index_rename (newibex, oldibex);
 ibex_failed:
-	camel_exception_setv (
-		ex, CAMEL_EXCEPTION_SYSTEM,
+	g_set_error (
+		error, G_IO_ERROR,
+		g_io_error_from_errno (errno),
 		_("Could not rename '%s': %s"),
 		old, g_strerror (errno));
 
-	g_free(newibex);
-	g_free(oldibex);
+	g_free (newibex);
+	g_free (oldibex);
 
 	if (folder)
 		g_object_unref (folder);
@@ -433,12 +457,12 @@ ibex_failed:
 
 /* default implementation, only delete metadata */
 static gboolean
-delete_folder (CamelStore *store,
-               const gchar *folder_name,
-               CamelException *ex)
+local_store_delete_folder_sync (CamelStore *store,
+                                const gchar *folder_name,
+                                GCancellable *cancellable,
+                                GError **error)
 {
 	CamelFolderInfo *fi;
-	CamelException lex;
 	CamelFolder *lf;
 	gchar *name;
 	gchar *str;
@@ -446,20 +470,20 @@ delete_folder (CamelStore *store,
 	/* remove metadata only */
 	name = g_strdup_printf("%s%s", CAMEL_LOCAL_STORE(store)->toplevel_dir, folder_name);
 	str = g_strdup_printf("%s.ibex", name);
-	if (camel_text_index_remove(str) == -1 && errno != ENOENT) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+	if (camel_text_index_remove (str) == -1 && errno != ENOENT) {
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
 			_("Could not delete folder index file '%s': %s"),
 			str, g_strerror (errno));
-		g_free(str);
+		g_free (str);
 		g_free (name);
 		return FALSE;
 	}
-	g_free(str);
+	g_free (str);
 
 	str = NULL;
-	camel_exception_init (&lex);
-	if ((lf = camel_store_get_folder (store, folder_name, 0, &lex))) {
+	if ((lf = camel_store_get_folder_sync (store, folder_name, 0, cancellable, NULL))) {
 		CamelObject *object = CAMEL_OBJECT (lf);
 		const gchar *state_filename;
 
@@ -469,16 +493,15 @@ delete_folder (CamelStore *store,
 		camel_object_set_state_filename (object, NULL);
 
 		g_object_unref (lf);
-	} else {
-		camel_exception_clear (&lex);
 	}
 
 	if (str == NULL)
 		str = g_strdup_printf ("%s.cmeta", name);
 
 	if (g_unlink (str) == -1 && errno != ENOENT) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+		g_set_error (
+			error, G_IO_ERROR,
+			g_io_error_from_errno (errno),
 			_("Could not delete folder meta file '%s': %s"),
 			str, g_strerror (errno));
 		g_free (name);
@@ -493,7 +516,7 @@ delete_folder (CamelStore *store,
 	fi->full_name = g_strdup (folder_name);
 	fi->name = g_path_get_basename (folder_name);
 	fi->uri = g_strdup_printf ("%s:%s#%s", ((CamelService *) store)->url->protocol,
-				   CAMEL_LOCAL_STORE(store)->toplevel_dir, folder_name);
+				   CAMEL_LOCAL_STORE (store)->toplevel_dir, folder_name);
 	fi->unread = -1;
 
 	camel_store_folder_deleted (store, fi);
@@ -503,19 +526,19 @@ delete_folder (CamelStore *store,
 }
 
 static gchar *
-local_get_full_path(CamelLocalStore *ls, const gchar *full_name)
+local_get_full_path (CamelLocalStore *ls, const gchar *full_name)
 {
 	return g_strdup_printf("%s%s", ls->toplevel_dir, full_name);
 }
 
 static gchar *
-local_get_meta_path(CamelLocalStore *ls, const gchar *full_name, const gchar *ext)
+local_get_meta_path (CamelLocalStore *ls, const gchar *full_name, const gchar *ext)
 {
 	return g_strdup_printf("%s%s%s", ls->toplevel_dir, full_name, ext);
 }
 
 static gboolean
-local_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelException *ex)
+local_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, GError **error)
 {
 	/* any local folder can be refreshed */
 	return TRUE;
