@@ -34,6 +34,7 @@
 
 #include "e-name-selector-entry.h"
 
+/* backward-compatibility cruft */
 #include "gtk-compat.h"
 
 #define E_NAME_SELECTOR_ENTRY_GET_PRIVATE(obj) \
@@ -106,6 +107,11 @@ name_selector_entry_dispose (GObject *object)
 
 	priv = E_NAME_SELECTOR_ENTRY_GET_PRIVATE (object);
 
+	if (priv->attr_list != NULL) {
+		pango_attr_list_unref (priv->attr_list);
+		priv->attr_list = NULL;
+	}
+
 	if (priv->entry_completion) {
 		g_object_unref (priv->entry_completion);
 		priv->entry_completion = NULL;
@@ -116,6 +122,16 @@ name_selector_entry_dispose (GObject *object)
 		priv->destination_store = NULL;
 	}
 
+	if (priv->email_generator) {
+		g_object_unref (priv->email_generator);
+		priv->email_generator = NULL;
+	}
+
+	if (priv->contact_store) {
+		g_object_unref (priv->contact_store);
+		priv->contact_store = NULL;
+	}
+
 	g_slist_foreach (priv->user_query_fields, (GFunc)g_free, NULL);
 	g_slist_free (priv->user_query_fields);
 	priv->user_query_fields = NULL;
@@ -280,7 +296,7 @@ get_utf8_string_context (const gchar *string, gint position, gunichar *unichars,
 		gint char_pos = position - gap + i;
 
 		if (char_pos < 0 || char_pos >= len) {
-			unichars [i] = '\0';
+			unichars[i] = '\0';
 			continue;
 		}
 
@@ -289,7 +305,7 @@ get_utf8_string_context (const gchar *string, gint position, gunichar *unichars,
 		else
 			p = g_utf8_offset_to_pointer (string, char_pos);
 
-		unichars [i] = g_utf8_get_char (p);
+		unichars[i] = g_utf8_get_char (p);
 	}
 }
 
@@ -481,7 +497,7 @@ name_style_query (const gchar *field, const gchar *value)
 
 	strv = g_strsplit (spaced_str, " ", 0);
 
-	if (strv [0] && strv [1]) {
+	if (strv[0] && strv[1]) {
 		g_string_append (out, "(or ");
 		comma_str = g_strjoinv (", ", strv);
 	} else {
@@ -638,8 +654,8 @@ get_entry_substring (ENameSelectorEntry *name_selector_entry, gint range_start,
 static gint
 utf8_casefold_collate_len (const gchar *str1, const gchar *str2, gint len)
 {
-	gchar *s1 = g_utf8_casefold(str1, len);
-	gchar *s2 = g_utf8_casefold(str2, len);
+	gchar *s1 = g_utf8_casefold (str1, len);
+	gchar *s2 = g_utf8_casefold (str2, len);
 	gint rv;
 
 	rv = g_utf8_collate (s1, s2);
@@ -695,7 +711,7 @@ static gboolean
 contact_match_cue (EContact *contact, const gchar *cue_str,
 		   EContactField *matched_field, gint *matched_field_rank)
 {
-	EContactField  fields [] = { E_CONTACT_FULL_NAME, E_CONTACT_NICKNAME, E_CONTACT_FILE_AS,
+	EContactField  fields[] = { E_CONTACT_FULL_NAME, E_CONTACT_NICKNAME, E_CONTACT_FILE_AS,
 				     E_CONTACT_EMAIL_1, E_CONTACT_EMAIL_2, E_CONTACT_EMAIL_3,
 				     E_CONTACT_EMAIL_4 };
 	gchar         *email;
@@ -725,11 +741,11 @@ contact_match_cue (EContact *contact, const gchar *cue_str,
 
 		/* Don't match e-mail addresses in contact lists */
 		if (e_contact_get (contact, E_CONTACT_IS_LIST) &&
-		    fields [i] >= E_CONTACT_FIRST_EMAIL_ID &&
-		    fields [i] <= E_CONTACT_LAST_EMAIL_ID)
+		    fields[i] >= E_CONTACT_FIRST_EMAIL_ID &&
+		    fields[i] <= E_CONTACT_LAST_EMAIL_ID)
 			continue;
 
-		value = e_contact_get (contact, fields [i]);
+		value = e_contact_get (contact, fields[i]);
 		if (!value)
 			continue;
 
@@ -919,8 +935,8 @@ type_ahead_complete (ENameSelectorEntry *name_selector_entry)
 		gint i;
 
 		/* keep character's case as user types */
-		for (i = 0; textrep [i] && cue_str [i]; i++)
-			textrep [i] = cue_str [i];
+		for (i = 0; textrep[i] && cue_str[i]; i++)
+			textrep[i] = cue_str[i];
 
 		gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), range_start, range_end);
 		gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), textrep, -1, &pos);
@@ -1145,13 +1161,37 @@ remove_destination_by_index (ENameSelectorEntry *name_selector_entry, gint index
 	}
 }
 
+static void
+post_insert_update (ENameSelectorEntry *name_selector_entry,
+                    gint position)
+{
+	const gchar *text;
+	glong length;
+
+	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
+	length = g_utf8_strlen (text, -1);
+	text = g_utf8_next_char (text);
+
+	if (*text == '\0') {
+		/* First and only character, create initial destination. */
+		insert_destination_at_position (name_selector_entry, 0);
+	} else {
+		/* Modified an existing destination. */
+		modify_destination_at_position (name_selector_entry, position);
+	}
+
+	/* If editing within the string, regenerate attributes. */
+	if (position < length)
+		generate_attribute_list (name_selector_entry);
+}
+
 /* Returns the number of characters inserted */
 static gint
 insert_unichar (ENameSelectorEntry *name_selector_entry, gint *pos, gunichar c)
 {
 	const gchar *text;
-	gunichar     str_context [4];
-	gchar        buf [7];
+	gunichar     str_context[4];
+	gchar        buf[7];
 	gint         len;
 
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
@@ -1161,7 +1201,7 @@ insert_unichar (ENameSelectorEntry *name_selector_entry, gint *pos, gunichar c)
 	 * - Before or after another space.
 	 * - At start of string. */
 
-	if (c == ' ' && (str_context [1] == ' ' || str_context [1] == '\0' || str_context [2] == ' '))
+	if (c == ' ' && (str_context[1] == ' ' || str_context[1] == '\0' || str_context[2] == ' '))
 		return 0;
 
 	/* Comma is not allowed:
@@ -1174,7 +1214,7 @@ insert_unichar (ENameSelectorEntry *name_selector_entry, gint *pos, gunichar c)
 		gboolean     at_start = FALSE;
 		gboolean     at_end   = FALSE;
 
-		if (str_context [1] == ',' || str_context [1] == '\0')
+		if (str_context[1] == ',' || str_context[1] == '\0')
 			return 0;
 
 		/* We do this so we can avoid disturbing destinations with completed contacts
@@ -1217,25 +1257,11 @@ insert_unichar (ENameSelectorEntry *name_selector_entry, gint *pos, gunichar c)
 	/* Generic case. Allowed spaces also end up here. */
 
 	len = g_unichar_to_utf8 (c, buf);
-	buf [len] = '\0';
+	buf[len] = '\0';
 
 	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), buf, -1, pos);
 
-	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
-	len = g_utf8_strlen (text, -1);
-	text = g_utf8_next_char (text);
-
-	if (!*text) {
-		/* First and only character so far, create initial destination */
-		insert_destination_at_position (name_selector_entry, 0);
-	} else {
-		/* Modified existing destination */
-		modify_destination_at_position (name_selector_entry, *pos);
-	}
-
-	/* If editing within the string, we need to regenerate attributes */
-	if (*pos < len)
-		generate_attribute_list (name_selector_entry);
+	post_insert_update (name_selector_entry, *pos);
 
 	return 1;
 }
@@ -1244,19 +1270,40 @@ static void
 user_insert_text (ENameSelectorEntry *name_selector_entry, gchar *new_text,
 		  gint new_text_length, gint *position, gpointer user_data)
 {
-	gchar *p;
-	gint   chars_inserted = 0;
+	gint chars_inserted = 0;
+	gboolean fast_insert;
 
 	g_signal_handlers_block_by_func (name_selector_entry, user_insert_text, name_selector_entry);
 	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 
-	/* Apply some rules as to where spaces and commas can be inserted, and insert
-	 * a trailing space after comma. */
+	fast_insert =
+		(g_utf8_strchr (new_text, new_text_length, ' ') == NULL) &&
+		(g_utf8_strchr (new_text, new_text_length, ',') == NULL);
 
-	for (p = new_text; *p; p = g_utf8_next_char (p)) {
-		gunichar c = g_utf8_get_char (p);
-		insert_unichar (name_selector_entry, position, c);
-		chars_inserted++;
+	/* If the text to insert does not contain spaces or commas,
+	 * insert all of it at once.  This avoids confusing on-going
+	 * input method behavior. */
+	if (fast_insert) {
+		gint old_position = *position;
+
+		gtk_editable_insert_text (
+			GTK_EDITABLE (name_selector_entry),
+			new_text, new_text_length, position);
+
+		chars_inserted = *position - old_position;
+		if (chars_inserted > 0)
+			post_insert_update (name_selector_entry, *position);
+
+	/* Otherwise, apply some rules as to where spaces and commas
+	 * can be inserted, and insert a trailing space after comma. */
+	} else {
+		const gchar *cp;
+
+		for (cp = new_text; *cp; cp = g_utf8_next_char (cp)) {
+			gunichar uc = g_utf8_get_char (cp);
+			insert_unichar (name_selector_entry, position, uc);
+			chars_inserted++;
+		}
 	}
 
 	if (chars_inserted >= 1) {
@@ -1278,10 +1325,10 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	const gchar *text;
 	gint         index_start, index_end;
 	gint	     selection_start, selection_end;
-	gunichar     str_context [2], str_b_context [2];
+	gunichar     str_context[2], str_b_context[2];
 	gint         len;
 	gint         i;
-	gboolean     already_selected = FALSE, del_space = FALSE, del_comma = FALSE;
+	gboolean     del_space = FALSE, del_comma = FALSE;
 
 	if (start_pos == end_pos)
 		return;
@@ -1289,12 +1336,9 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	len = g_utf8_strlen (text, -1);
 
-	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (name_selector_entry),
-					       &selection_start,
-					       &selection_end))
-		if ((g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end)) == 0) ||
-		    (g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end)) == ','))
-			already_selected = TRUE;
+	gtk_editable_get_selection_bounds (
+		GTK_EDITABLE (name_selector_entry),
+		&selection_start, &selection_end);
 
 	get_utf8_string_context (text, start_pos, str_context, 2);
 	get_utf8_string_context (text, end_pos, str_b_context, 2);
@@ -1370,6 +1414,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 			/* If we are at the beginning or removing junk space, let us ignore it */
 			del_space = TRUE;
 		}
+		g_free (c);
 	} else	if (end_pos == start_pos +1 &&  index_end == index_start+1) {
 		/* We could be just deleting the empty text */
 		gchar *c;
@@ -1381,6 +1426,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 			/* If we are at the beginning or removing junk space, let us ignore it */
 			del_comma = TRUE;
 		}
+		g_free (c);
 	}
 
 	if (del_comma) {
@@ -1433,7 +1479,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	 addresses.
 	*/
 
-	if (str_b_context [1] == '"') {
+	if (str_b_context[1] == '"') {
 		const gchar *p;
 		gint j;
 		p = text + end_pos;
@@ -1448,7 +1494,7 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 
 	/* Let model know about changes */
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
-	if (!*text || strlen(text) <= 0) {
+	if (!*text || strlen (text) <= 0) {
 		/* If the entry was completely cleared, remove the initial destination too */
 		remove_destination_by_index (name_selector_entry, 0);
 		generate_attribute_list (name_selector_entry);
@@ -1497,7 +1543,10 @@ completion_match_selected (ENameSelectorEntry *name_selector_entry, GtkTreeModel
 	destination = find_destination_at_position (name_selector_entry, cursor_pos);
 	e_destination_set_contact (destination, contact, email_n);
 	sync_destination_at_position (name_selector_entry, cursor_pos, &cursor_pos);
-	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ",", -1, &cursor_pos);
+
+	g_signal_handlers_block_by_func (name_selector_entry, user_insert_text, name_selector_entry);
+	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ", ", -1, &cursor_pos);
+	g_signal_handlers_unblock_by_func (name_selector_entry, user_insert_text, name_selector_entry);
 
 	/* Place cursor at end of address */
 
@@ -1571,6 +1620,7 @@ entry_activate (ENameSelectorEntry *name_selector_entry)
 			range_end = range_end+2;
 
 		}
+		g_free (str_context);
 	}
 
 	gtk_editable_set_position (GTK_EDITABLE (name_selector_entry), range_end);
@@ -1920,6 +1970,40 @@ setup_contact_store (ENameSelectorEntry *name_selector_entry)
 	}
 }
 
+static void
+book_loaded_cb (ESource *source,
+                GAsyncResult *result,
+                ENameSelectorEntry *name_selector_entry)
+{
+	EBook *book;
+	EContactStore *store;
+	GError *error = NULL;
+
+	book = e_load_book_source_finish (source, result, &error);
+
+	if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		g_warn_if_fail (book == NULL);
+		g_error_free (error);
+		goto exit;
+	}
+
+	if (error != NULL) {
+		g_warning ("%s", error->message);
+		g_warn_if_fail (book == NULL);
+		g_error_free (error);
+		goto exit;
+	}
+
+	g_return_if_fail (E_IS_BOOK (book));
+
+	store = name_selector_entry->priv->contact_store;
+	e_contact_store_add_book (store, book);
+	g_object_unref (book);
+
+exit:
+	g_object_unref (name_selector_entry);
+}
+
 static void
 setup_default_contact_store (ENameSelectorEntry *name_selector_entry)
 {
@@ -1939,8 +2023,7 @@ setup_default_contact_store (ENameSelectorEntry *name_selector_entry)
 		GSList       *m;
 
 		for (m = sources; m; m = g_slist_next (m)) {
-			ESource     *source = m->data;
-			EBook       *book;
+			ESource *source = m->data;
 			const gchar *completion;
 
 			/* Skip non-completion sources */
@@ -1948,12 +2031,10 @@ setup_default_contact_store (ENameSelectorEntry *name_selector_entry)
 			if (!completion || g_ascii_strcasecmp (completion, "true"))
 				continue;
 
-			book = e_load_book_source_ex (source, NULL, NULL);
-			if (!book)
-				continue;
-
-			e_contact_store_add_book (name_selector_entry->priv->contact_store, book);
-			g_object_unref (book);
+			e_load_book_source_async (
+				source, NULL, NULL,
+				(GAsyncReadyCallback) book_loaded_cb,
+				g_object_ref (name_selector_entry));
 		}
 	}
 
@@ -1969,7 +2050,7 @@ destination_row_changed (ENameSelectorEntry *name_selector_entry, GtkTreePath *p
 	gint          range_start, range_end;
 	gint          n;
 
-	n = gtk_tree_path_get_indices (path) [0];
+	n = gtk_tree_path_get_indices (path)[0];
 	destination = e_destination_store_get_destination (name_selector_entry->priv->destination_store, iter);
 
 	if (!destination)
@@ -2011,7 +2092,7 @@ destination_row_inserted (ENameSelectorEntry *name_selector_entry, GtkTreePath *
 	gint          insert_pos;
 	gint          n;
 
-	n = gtk_tree_path_get_indices (path) [0];
+	n = gtk_tree_path_get_indices (path)[0];
 	destination = e_destination_store_get_destination (name_selector_entry->priv->destination_store, iter);
 
 	g_assert (n >= 0);
@@ -2062,7 +2143,7 @@ destination_row_deleted (ENameSelectorEntry *name_selector_entry, GtkTreePath *p
 	gchar       *p0;
 	gint         n;
 
-	n = gtk_tree_path_get_indices (path) [0];
+	n = gtk_tree_path_get_indices (path)[0];
 	g_assert (n >= 0);
 
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
@@ -2214,7 +2295,7 @@ find_book_by_contact (GList *books, const gchar *contact_uid)
 }
 
 static void
-editor_closed_cb (GtkObject *editor, gpointer data)
+editor_closed_cb (GtkWidget *editor, gpointer data)
 {
 	EContact *contact;
 	gchar *contact_uid;
@@ -2243,8 +2324,8 @@ editor_closed_cb (GtkObject *editor, gpointer data)
 	if (!book)
 		return;
 
-	result = e_book_get_contact(book, contact_uid, &contact, NULL);
-	email_num = e_destination_get_email_num(destination);
+	result = e_book_get_contact (book, contact_uid, &contact, NULL);
+	email_num = e_destination_get_email_num (destination);
 	e_destination_set_contact (destination, contact, email_num);
 
 	g_free (contact_uid);
@@ -2263,7 +2344,7 @@ popup_activate_inline_expand (ENameSelectorEntry *name_selector_entry, GtkWidget
 
 	position = GPOINTER_TO_INT(g_object_get_data ((GObject *)name_selector_entry, "index"));
 
-	email_list = e_destination_get_address(destination);
+	email_list = e_destination_get_address (destination);
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	get_range_at_position (text, position, &start, &end);
 
@@ -2374,7 +2455,7 @@ popup_activate_cut (ENameSelectorEntry *name_selector_entry, GtkWidget *menu_ite
 	GtkClipboard *clipboard;
 
 	destination = name_selector_entry->priv->popup_destination;
-	contact_email =e_destination_get_address(destination);
+	contact_email =e_destination_get_address (destination);
 
 	g_signal_handlers_block_by_func (name_selector_entry, user_insert_text, name_selector_entry);
 	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
@@ -2403,7 +2484,7 @@ popup_activate_copy (ENameSelectorEntry *name_selector_entry, GtkWidget *menu_it
 	GtkClipboard *clipboard;
 
 	destination = name_selector_entry->priv->popup_destination;
-	contact_email = e_destination_get_address(destination);
+	contact_email = e_destination_get_address (destination);
 
 	g_signal_handlers_block_by_func (name_selector_entry, user_insert_text, name_selector_entry);
 	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
@@ -2508,7 +2589,7 @@ populate_popup (ENameSelectorEntry *name_selector_entry, GtkMenu *menu)
 			show_menu = TRUE;
 
 			if (length > 1) {
-				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !e_destination_is_ignored(dest));
+				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !e_destination_is_ignored (dest));
 				g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (popup_activate_list),
 							  dest);
 			}
@@ -2611,30 +2692,43 @@ populate_popup (ENameSelectorEntry *name_selector_entry, GtkMenu *menu)
 }
 
 static void
-copy_or_cut_clipboard (ENameSelectorEntry *name_selector_entry, gboolean is_cut)
+copy_or_cut_clipboard (ENameSelectorEntry *name_selector_entry,
+                       gboolean is_cut)
 {
-	gint i, start = 0, end = 0;
-	const gchar *text;
+	GtkClipboard *clipboard;
+	GtkEditable *editable;
+	const gchar *text, *cp;
 	GHashTable *hash;
 	GHashTableIter iter;
 	gpointer key, value;
 	GString *addresses;
+	gint ii, start, end;
+	gunichar uc;
 
-	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
+	editable = GTK_EDITABLE (name_selector_entry);
+	text = gtk_entry_get_text (GTK_ENTRY (editable));
 
-	if (!gtk_editable_get_selection_bounds (GTK_EDITABLE (name_selector_entry), &start, &end)) {
-		start = gtk_editable_get_position (GTK_EDITABLE (name_selector_entry));
-		end = start;
-	}
-
-	/* do nothing when there is nothing selected */
-	if (start == end)
+	if (!gtk_editable_get_selection_bounds (editable, &start, &end))
 		return;
 
-	hash = g_hash_table_new (g_direct_hash, g_direct_equal);
-	for (i = start; i <= end; i++) {
-		gint index = get_index_at_position (text, i);
+	g_return_if_fail (end > start);
 
+	hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+
+	ii = end;
+	cp = g_utf8_offset_to_pointer (text, end);
+	uc = g_utf8_get_char (cp);
+
+	/* Exclude trailing whitespace and commas. */
+	while (ii >= start && (uc == ',' || g_unichar_isspace (uc))) {
+		cp = g_utf8_prev_char (cp);
+		uc = g_utf8_get_char (cp);
+		ii--;
+	}
+
+	/* Determine the index of each remaining character. */
+	while (ii >= start) {
+		gint index = get_index_at_position (text, ii--);
 		g_hash_table_insert (hash, GINT_TO_POINTER (index), NULL);
 	}
 
@@ -2676,11 +2770,13 @@ copy_or_cut_clipboard (ENameSelectorEntry *name_selector_entry, gboolean is_cut)
 	}
 
 	if (is_cut)
-		gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), start, end);
+		gtk_editable_delete_text (editable, start, end);
 
 	g_hash_table_unref (hash);
 
-	gtk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (name_selector_entry), GDK_SELECTION_CLIPBOARD), addresses->str, -1);
+	clipboard = gtk_widget_get_clipboard (
+		GTK_WIDGET (name_selector_entry), GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clipboard, addresses->str, -1);
 
 	g_string_free (addresses, TRUE);
 }
@@ -2716,7 +2812,7 @@ e_name_selector_entry_init (ENameSelectorEntry *name_selector_entry)
 	}
 
 	/* read minimum_query_length from gconf*/
-	gconf = gconf_client_get_default();
+	gconf = gconf_client_get_default ();
 	if (COMPLETION_CUE_MIN_LEN == 0) {
 	  if ((COMPLETION_CUE_MIN_LEN = gconf_client_get_int (gconf, MINIMUM_QUERY_LENGTH, NULL)))
 		;
@@ -2881,6 +2977,11 @@ e_name_selector_entry_set_destination_store  (ENameSelectorEntry *name_selector_
 	setup_destination_store (name_selector_entry);
 }
 
+/**
+ * e_name_selector_entry_get_popup_destination:
+ *
+ * Since: 2.32
+ **/
 EDestination *
 e_name_selector_entry_get_popup_destination (ENameSelectorEntry *name_selector_entry)
 {
